//
//  ViewController.swift
//  TouchBar
//
//  Created by Chris Ricker on 10/28/16.
//  Copyright © 2016 Ray Wenderlich. All rights reserved.
//

import Cocoa

class ViewController: NSViewController {
  
  @IBOutlet var nameField: NSTextField!
  
  var visited = 0
  var rating = 0
  
  override func viewDidLoad() {
    super.viewDidLoad()
    if #available(OSX 10.12.1, *) {
      nameField.isAutomaticTextCompletionEnabled = false
    }
  }
  
  @IBAction func save(_ sender: Any) {
    willChangeValue(forKey: "rating")
    willChangeValue(forKey: "visited")
    rating = 0
    visited = 0
    nameField.stringValue = ""
    didChangeValue(forKey: "rating")
    didChangeValue(forKey: "visited")
  }
  
  @IBAction func changevisitedAmount(_ sender: NSSegmentedControl) {
    willChangeValue(forKey: "visited")
    switch sender.selectedSegment {
    case 0:
      if visited > 0 {
        visited -= 1
      }
    case 1:
      visited += 1
    default:
      break
    }
    didChangeValue(forKey: "visited")
  }
  
  @IBAction func changeRating(_ sender: NSSegmentedControl) {
    willChangeValue(forKey: "rating")
    switch sender.selectedSegment {
    case 0:
      if rating > 0 {
        rating -= 1
      }
    case 1:
      if rating < 4 {
        rating += 1
      }
    default:
      break
    }
    didChangeValue(forKey: "rating")
  }
  
}

// MARK: - Scrubber DataSource & Delegate
@available(OSX 10.12.1, *)
extension ViewController: NSScrubberDataSource, NSScrubberDelegate {
  
  func numberOfItems(for scrubber: NSScrubber) -> Int {
    return 5
  }
  
  func scrubber(_ scrubber: NSScrubber, viewForItemAt index: Int) -> NSScrubberItemView {
    let itemView = scrubber.makeItem(withIdentifier: "RatingScrubberItemIdentifier", owner: nil) as! NSScrubberTextItemView
    itemView.textField.stringValue = String(index)
    return itemView
  }
  
  func scrubber(_ scrubber: NSScrubber, didSelectItemAt index: Int) {
    willChangeValue(forKey: "rating")
    rating = index
    didChangeValue(forKey: "rating")
  }
  
}

// MARK: - TouchBar Delegate
@available(OSX 10.12.1, *)
extension ViewController: NSTouchBarDelegate {
  
  override func makeTouchBar() -> NSTouchBar? {
    let touchBar = NSTouchBar()
    touchBar.delegate = self
    touchBar.customizationIdentifier = .ratingBar
    touchBar.defaultItemIdentifiers = [.infoLabelItem, .flexibleSpace, .ratingLabel, .ratingScrubber, .flexibleSpace, .visitedLabelItem, .visitedItem, .visitSegmentedItem, .flexibleSpace, .saveItem]
    touchBar.customizationAllowedItemIdentifiers = [.infoLabelItem]
    return touchBar
  }
  
  func touchBar(_ touchBar: NSTouchBar, makeItemForIdentifier identifier: NSTouchBarItemIdentifier) -> NSTouchBarItem? {
    switch identifier {
    case NSTouchBarItemIdentifier.infoLabelItem:
      let customViewItem = NSCustomTouchBarItem(identifier: identifier)
      customViewItem.view = NSTextField(labelWithString: "🌎 📓")
      return customViewItem
    case NSTouchBarItemIdentifier.ratingLabel:
      let customViewItem = NSCustomTouchBarItem(identifier: identifier)
      customViewItem.view = NSTextField(labelWithString: "Rating")
      return customViewItem
    case NSTouchBarItemIdentifier.ratingScrubber:
      let scrubberItem = NSCustomTouchBarItem(identifier: identifier)
      
      let scrubber = NSScrubber()
      scrubber.scrubberLayout = NSScrubberFlowLayout()
      scrubber.register(NSScrubberTextItemView.self, forItemIdentifier: "RatingScrubberItemIdentifier")
      scrubber.mode = .fixed
      scrubber.selectionBackgroundStyle = .roundedBackground
      scrubber.delegate = self
      scrubber.dataSource = self
      scrubberItem.view = scrubber
      scrubber.bind("selectedIndex", to: self, withKeyPath: #keyPath(rating), options: nil)
      
      return scrubberItem
    case NSTouchBarItemIdentifier.visitedLabelItem:
      // 1
      let customViewItem = NSCustomTouchBarItem(identifier: identifier)
      customViewItem.view = NSTextField(labelWithString: "Times Visited")
      return customViewItem
    case NSTouchBarItemIdentifier.visitedItem:
      // 2
      let customViewItem = NSCustomTouchBarItem(identifier: identifier)
      customViewItem.view = NSTextField(labelWithString: "--")
      customViewItem.view.bind("value", to: self, withKeyPath: #keyPath(visited), options: nil)
      return customViewItem
    case NSTouchBarItemIdentifier.visitSegmentedItem:
      // 3
      let customActionItem = NSCustomTouchBarItem(identifier: identifier)
      let segmentedControl = NSSegmentedControl(images: [NSImage(named: NSImageNameRemoveTemplate)!, NSImage(named: NSImageNameAddTemplate)!], trackingMode: .momentary, target: self, action: #selector(changevisitedAmount(_:)))
      segmentedControl.setWidth(40, forSegment: 0)
      segmentedControl.setWidth(40, forSegment: 1)
      customActionItem.view = segmentedControl
      return customActionItem
    case NSTouchBarItemIdentifier.saveItem:
      let saveItem = NSCustomTouchBarItem(identifier: identifier)
      let button = NSButton(title: "Save", target: self, action: #selector(save(_:)))
      button.bezelColor = NSColor(red:0.35, green:0.61, blue:0.35, alpha:1.00)
      saveItem.view = button
      return saveItem
    default:
      return nil
    }
  }
}
